local function getUnitBuffs(buffs, unit)
    for i=1,40 do
        local name, _, _, _, _, _, _, _, _, spellID, _, _, _, _, _, value1, value2, value3 = UnitAura(unit, i);
        if (name ~= nil and spellID ~= nil) then
            table.insert(buffs, 1, spellID);
        end
    end
end

local function findMountFromBuffs(buffs, mounts)
    local mountID = nil;
    local mountName = nil;
    local mountOwned = nil;
    for bi, buff in pairs(buffs) do
        for mi, mount in ipairs(mounts) do
            if (mount[2] == buff) then
                mountID = mount[3];
                mountName = mount[1];
                mountOwned = mount[4];
            end
        end
    end
    return mountID, mountName, mountOwned;
end

local SOURCE_BLIZZARD_STORE = 10;

-- certain mounts are banned for fair battle
-- for example: class mounts, store mounts, mounts that require a profession
local BANNED_MOUNT_IDS =
{
	171844,		-- Dustmane Direwolf (leatherworking)
	213339,		-- Great Northern Elderhorn (leatherworking)

	44153,		-- Flying Machine (engineering)
	44151,		-- Turbo-Charged Flying Machine (engineering)

	61309,		-- Magnificent Flying Carpet (tailoring)
	169952,		-- Creeping Carpet (tailoring)
	61451,		-- Flying Carpet	 (tailoring)
	75596,		-- Frosty Flying Carpet (tailoring)

	73629,		-- Exarch's Elekk				
	270562,		-- Darkforge Ram	
	270564,		-- Dawnforge Ram
	69820,		-- Sunwalker Kodo
	34769,		-- Thalassian Warhorse
	13819,		-- Warhorse
	73630,		-- Great Exarch's Elekk
	23214,		-- Charger
	34767,		-- Thalassian Charger
	69826,		-- Great Sunwalker Kodo
	66906,		-- Argent Charger
	231435,		-- Highlord's Golden Charger
	231589,		-- Highlord's Valorous Charger
	231587,		-- Highlord's Vengeful Charger
	231588,		-- Highlord's Vigilant Charger
	5784,		-- Felsteed
	23161,		-- Dreadsteed
	238454,		-- Netherlord's Accursed Wrathsteed
	238452,		-- Netherlord's Brimstone Wrathsteed
	232412,		-- Netherlord's Chaotic Wrathsteed		
	48778,		-- Acherus Deathcharger
	54729,		-- Winger Steed of the Ebon Blade
	229387,		-- Deathlord's Vilebrood Vanquisher
	200175,		-- Felsaber
	229417,		-- Slayer's Felbroken Shrieker
	229439,		-- Huntmaster's Dire Wolfhawk
	229438,		-- Huntmaster's Fierce Wolfhawk
	229386,		-- Huntmaster's Loyal Wolfhawk
	229376,		-- Archmage's Prismatic Disc
	229385,		-- Ban-Lu, Grandmaster's Companion
	229377,		-- High Priest's Lightsworn Seeker
	231524,		-- Shadowblade's Baneful Omen
	231525,		-- Shadowblade's Crimson Omen
	231523,		-- Shadowblade's Lethal Omen
	231434,		-- Shadowblade's Murderous Omen
	231442,		-- Farseer's Raging Tempest
	229388,		-- Battlelord's Bloodthirsty War Wyrm	
}

local function isMountBanned(sourceType, spellID)

	-- store mounts are all banned
	if (sourceType == SOURCE_BLIZZARD_STORE) then
		return true;
	end

	for i, mountID in pairs(BANNED_MOUNT_IDS) do
		if (mountID == spellID) then
			return true;
		end
	end

	return false;
end



function MountBattle()    
    if (UnitExists("target")) then
        
        -- find owned mounts that can be mounted by this character
        local allMounts = {};
        local myMounts = {};
        
        local mountIDs = C_MountJournal.GetMountIDs();
		for i, mountID in ipairs(mountIDs) do
			local name, spellID, _, _, usable, sourceType, _, _, _, hideOnChar, isCollected = C_MountJournal.GetMountInfoByID(mountID);
            
            local canMount = false;
			if (isCollected and hideOnChar ~= true and usable) then
                canMount = true;

				if (isMountBanned(sourceType, spellID) ~= true) then
					-- push into my mounts table for random summon
                    table.insert(myMounts, 1, mountID);
                end
			end            
            
            local mount = {name, spellID, mountID, canMount};
            table.insert(allMounts, 1, mount);
		end
        
        -- find all buffs of the target player
        local targetBuffs = {};    
        local playerBuffs = {};
        getUnitBuffs(targetBuffs, "target");
        getUnitBuffs(playerBuffs, "player");
        
        local targetMountID, targetMountName, targetMountOwned = findMountFromBuffs(targetBuffs, allMounts);
        local playerMountID, playerMountName, playerMountOwned = findMountFromBuffs(playerBuffs, allMounts);
        
        if (targetMountID ~= nil) then
            
            -- mount it if we have it and we aren't already mounting it
            if (targetMountOwned and targetMountID ~= playerMountID) then
                --print("Mounting " .. targetMountID);
                C_MountJournal.SummonByID(targetMountID);
            else
                -- otherwise summon a random mount
                --print("I don't have "..targetMountName..", "..UnitName("target").." :(");
            
                local randomID = math.random(1, #myMounts);                
                C_MountJournal.SummonByID(myMounts[randomID]);
            end
        else
            UIErrorsFrame:AddMessage("Target is not mounted.", 1.0, 0.0, 0.0);
        end
    else
        UIErrorsFrame:AddMessage("You have no target.", 1.0, 0.0, 0.0);
    end
end